
# q-expansion of newform 7200.2.h.b, downloaded from the LMFDB on 12 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7200
weight = 2
poly_data = [1, 0, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 1, 0], 1], [[0, 1, 0, 1], 1], [[0, 1, 0, -1], 1]]

hecke_ring_character_values = [[6751, [-1, 0, 0, 0]], [901, [1, 0, 0, 0]], [6401, [-1, 0, 0, 0]], [577, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 1, 1, 0], [-4, 0, 0, -1], [1, 0, 0, 4], [0, -2, -1, 0], [0, 3, 3, 0], [4, 0, 0, -1], [0, -6, -3, 0], [0, -1, -3, 0], [4, 0, 0, 2], [0, -2, -2, 0], [0, -5, 1, 0], [-6, 0, 0, -1], [0, -2, 2, 0], [-2, 0, 0, 5], [1, 0, 0, -2], [0, 1, -3, 0], [6, 0, 0, 4], [-8, 0, 0, -6], [0, 12, 0, 0], [-6, 0, 0, 1], [0, 12, 0, 0], [9, 0, 0, 0], [0, 2, -2, 0], [0, -2, -4, 0], [6, 0, 0, 8], [11, 0, 0, -2], [0, 4, 4, 0], [0, 8, -6, 0], [6, 0, 0, 0], [0, 16, -1, 0], [0, 4, -10, 0], [0, 12, -7, 0], [0, 1, 5, 0], [1, 0, 0, -6], [0, -17, -3, 0], [14, 0, 0, 4], [0, 6, 3, 0], [6, 0, 0, 13], [7, 0, 0, 0], [-12, 0, 0, 9], [-7, 0, 0, 6], [0, 8, -1, 0], [0, 17, -5, 0], [0, 5, -15, 0], [0, 5, 9, 0], [-18, 0, 0, 4], [17, 0, 0, 6], [0, -18, 2, 0], [-6, 0, 0, 0], [3, 0, 0, -12], [8, 0, 0, -8], [0, -12, 0, 0], [6, 0, 0, -12], [0, -14, -5, 0], [0, 4, 6, 0], [15, 0, 0, 0], [0, 18, -7, 0], [0, -23, 5, 0], [0, 0, 11, 0], [0, 3, 3, 0], [-6, 0, 0, 13], [-1, 0, 0, 14], [0, 0, -8, 0], [0, 24, 6, 0], [9, 0, 0, -6], [10, 0, 0, 13], [12, 0, 0, -12], [0, 10, 7, 0], [-14, 0, 0, -8], [0, 1, -13, 0], [-7, 0, 0, 0], [0, 11, 11, 0], [8, 0, 0, -16], [0, 6, -10, 0], [5, 0, 0, 0], [0, -18, -6, 0], [17, 0, 0, 2], [2, 0, 0, 20], [12, 0, 0, 6], [0, 0, 0, 1], [-9, 0, 0, -6], [0, 23, -11, 0], [-20, 0, 0, 8], [0, 8, -8, 0], [-6, 0, 0, 0], [0, -2, -14, 0], [0, 4, -26, 0], [0, 0, 0, 13], [14, 0, 0, -1], [0, 7, -9, 0], [-12, 0, 0, -8], [0, 1, -17, 0], [-2, 0, 0, -28], [0, 24, 7, 0], [0, -30, -3, 0], [0, -17, -3, 0], [5, 0, 0, 4], [0, 10, -12, 0], [0, 12, -15, 0], [-12, 0, 0, 19], [0, 6, 3, 0], [0, 3, 3, 0], [23, 0, 0, 0], [18, 0, 0, 4], [0, -10, 14, 0], [20, 0, 0, 4], [1, 0, 0, 26], [0, 12, 12, 0], [24, 0, 0, 12], [0, 30, -2, 0], [0, -15, 9, 0], [0, -13, -17, 0], [0, 16, 4, 0], [0, -28, 10, 0], [-12, 0, 0, 4], [0, -16, -7, 0], [10, 0, 0, 20], [8, 0, 0, -12], [8, 0, 0, -6], [0, -30, 13, 0], [-32, 0, 0, -4], [0, 22, 0, 0], [0, 30, 15, 0], [-1, 0, 0, 6], [8, 0, 0, -17], [0, 1, 5, 0], [-4, 0, 0, 8], [0, -8, 4, 0], [0, 0, 0, -12], [0, 16, 26, 0], [-5, 0, 0, -14], [0, 32, -8, 0], [-11, 0, 0, 14], [0, 6, 22, 0], [0, -31, -5, 0], [0, -8, -20, 0], [0, -30, 14, 0], [0, -7, -17, 0], [0, -24, -1, 0], [0, 35, 15, 0], [-38, 0, 0, 4], [6, 0, 0, 24], [-24, 0, 0, -15], [-1, 0, 0, 18], [0, -28, -4, 0], [0, -10, 0, 0], [44, 0, 0, -4], [-19, 0, 0, 2], [0, 0, 32, 0], [0, -3, 3, 0], [-2, 0, 0, -17], [0, 8, 8, 0], [-6, 0, 0, 23], [0, 5, 23, 0], [0, 18, -7, 0], [-31, 0, 0, 6], [0, 4, -13, 0], [36, 0, 0, 7], [0, 36, 4, 0], [0, -4, -24, 0], [22, 0, 0, -19], [0, 20, 25, 0], [-12, 0, 0, -4], [0, -7, 5, 0], [12, 0, 0, -24], [-5, 0, 0, 28], [0, 48, -7, 0], [-8, 0, 0, 1], [-23, 0, 0, 8], [-34, 0, 0, 5], [-1, 0, 0, -2], [0, -35, -15, 0], [0, -4, 20, 0], [0, 11, -7, 0], [0, -18, 23, 0], [0, 15, -27, 0], [-11, 0, 0, -6], [0, -39, 9, 0], [36, 0, 0, 4], [7, 0, 0, -28], [0, -12, -33, 0], [-46, 0, 0, -7], [0, -4, -19, 0], [-41, 0, 0, -12], [0, 13, -25, 0], [-9, 0, 0, -18], [-12, 0, 0, -29], [31, 0, 0, 10], [-6, 0, 0, 28], [0, -11, 3, 0], [0, -6, -21, 0], [8, 0, 0, 7], [0, -24, -15, 0], [-8, 0, 0, -2], [20, 0, 0, -20], [0, 0, 37, 0], [-40, 0, 0, -7], [0, 6, 26, 0], [0, -34, 12, 0], [-14, 0, 0, 8], [-32, 0, 0, 14], [24, 0, 0, -8], [0, 34, -25, 0], [0, -29, -15, 0], [-56, 0, 0, -1], [0, 0, 24, 0], [0, -11, 7, 0], [37, 0, 0, 20], [0, 16, 29, 0], [0, 5, -3, 0], [-4, 0, 0, -28], [30, 0, 0, -13], [0, 0, 0, 30], [0, 15, -15, 0], [0, -6, 3, 0], [10, 0, 0, 8], [0, 6, -39, 0], [-9, 0, 0, -24], [0, -49, -3, 0], [0, -8, -17, 0], [0, 28, -6, 0], [2, 0, 0, -8], [-39, 0, 0, 24], [0, -6, -25, 0], [30, 0, 0, -7], [0, -43, 7, 0], [-4, 0, 0, 29], [-3, 0, 0, -12], [0, -24, -6, 0], [0, 1, -31, 0], [0, -12, -13, 0], [0, 16, 18, 0], [0, 0, 0, -28], [-40, 0, 0, -6], [0, -34, -10, 0], [30, 0, 0, -5], [6, 0, 0, -32], [-4, 0, 0, 7], [0, -29, -21, 0], [0, 3, 9, 0], [-7, 0, 0, 2], [0, -4, 1, 0], [30, 0, 0, -3], [0, 19, 29, 0], [-48, 0, 0, 5], [0, 3, 33, 0], [30, 0, 0, -3], [-24, 0, 0, 0], [0, 6, 45, 0], [-24, 0, 0, -20], [-13, 0, 0, 10], [0, -14, -29, 0], [36, 0, 0, -8], [7, 0, 0, -6], [0, 32, -2, 0], [0, -26, 14, 0], [-64, 0, 0, -4], [0, 7, 3, 0], [-8, 0, 0, -31], [-16, 0, 0, 20], [-41, 0, 0, 24], [0, -20, 17, 0], [0, -6, 24, 0], [0, 8, 47, 0], [0, 12, -24, 0], [0, 16, -24, 0], [0, -60, 11, 0], [-25, 0, 0, 12], [0, -6, -24, 0], [20, 0, 0, 16], [0, -23, 15, 0], [19, 0, 0, -8], [0, -19, -21, 0], [30, 0, 0, -21], [7, 0, 0, 16], [3, 0, 0, 6], [-12, 0, 0, 4], [-30, 0, 0, 20], [0, 20, 8, 0], [-20, 0, 0, 13], [-19, 0, 0, 22], [0, -1, 47, 0], [6, 0, 0, -45], [39, 0, 0, -12], [0, 6, 18, 0], [0, 16, 18, 0], [0, -34, 2, 0], [0, 12, -25, 0], [16, 0, 0, 16], [0, 0, -36, 0], [24, 0, 0, 12], [25, 0, 0, 18], [0, -66, -2, 0], [0, -19, -13, 0], [0, 6, -30, 0], [-36, 0, 0, -25], [0, 40, 12, 0], [-8, 0, 0, -12], [0, -24, -40, 0], [0, -43, 21, 0], [-30, 0, 0, -27], [0, 8, 32, 0], [-7, 0, 0, -46], [-44, 0, 0, -16], [17, 0, 0, -2], [-54, 0, 0, 16], [50, 0, 0, -16], [0, 0, 0, 51], [0, 48, -12, 0], [0, -48, -20, 0], [0, -17, -37, 0], [-16, 0, 0, 5], [-3, 0, 0, -6], [36, 0, 0, -13], [4, 0, 0, 8], [-11, 0, 0, -44], [0, -6, -15, 0], [0, 6, 0, 0], [-57, 0, 0, 0], [0, -46, 26, 0], [0, -33, -21, 0], [0, 0, -17, 0], [8, 0, 0, 10], [0, 23, -39, 0], [0, 41, -27, 0], [-24, 0, 0, -31], [0, 2, 38, 0], [-25, 0, 0, -30], [0, 12, -1, 0], [0, -9, 15, 0], [12, 0, 0, -21], [0, -45, 9, 0], [4, 0, 0, 16], [-21, 0, 0, -36], [0, -18, -22, 0], [-35, 0, 0, 16], [0, -36, -12, 0], [-77, 0, 0, -10], [0, 46, -22, 0], [0, 78, 5, 0], [0, 9, 45, 0], [0, 78, -11, 0], [54, 0, 0, -5], [-31, 0, 0, -20], [0, 67, -15, 0], [-54, 0, 0, -24], [0, 0, 16, 0], [0, -37, -39, 0], [-4, 0, 0, -4], [0, 30, -27, 0], [0, -43, -1, 0], [-9, 0, 0, 42], [0, -24, 39, 0], [18, 0, 0, 28], [60, 0, 0, -21], [0, -54, 9, 0], [-24, 0, 0, 45], [-5, 0, 0, 6], [0, 60, -21, 0], [24, 0, 0, 49], [48, 0, 0, 0], [0, -16, -26, 0], [21, 0, 0, 18], [0, 48, -4, 0], [0, -22, 44, 0], [7, 0, 0, -26], [-52, 0, 0, 16], [0, 37, -5, 0], [4, 0, 0, 13], [0, -8, -56, 0], [0, 28, -44, 0], [-41, 0, 0, 24], [6, 0, 0, 16], [-18, 0, 0, -23], [15, 0, 0, 54], [0, -20, -16, 0], [15, 0, 0, 24], [0, 44, -19, 0], [0, -30, -11, 0], [0, 28, 4, 0], [0, 18, 26, 0], [0, 40, 8, 0], [-30, 0, 0, -5], [0, -13, -31, 0], [24, 0, 0, -24], [0, 16, 50, 0], [-24, 0, 0, -5], [21, 0, 0, 24], [0, -14, 50, 0], [-6, 0, 0, -44], [0, 35, -43, 0], [6, 0, 0, 33], [11, 0, 0, -10], [0, -23, -9, 0], [0, 54, 22, 0], [0, -27, -21, 0], [0, -60, 4, 0], [44, 0, 0, 30], [0, -54, 22, 0], [0, 15, -3, 0], [0, -18, -14, 0], [0, -18, 22, 0], [0, 20, -12, 0], [45, 0, 0, 12], [0, 50, -14, 0], [0, -37, -19, 0], [-12, 0, 0, 36], [5, 0, 0, 50], [0, 32, 28, 0], [-10, 0, 0, -55], [0, 19, -33, 0], [-27, 0, 0, -6], [0, -60, -20, 0], [-6, 0, 0, 27], [0, -37, 31, 0], [0, 30, -2, 0], [18, 0, 0, 24], [0, -60, -16, 0], [25, 0, 0, -30], [-30, 0, 0, 27], [-18, 0, 0, -37], [-64, 0, 0, 16], [0, 18, -14, 0], [-38, 0, 0, -4], [0, -42, -18, 0], [0, -23, 23, 0], [20, 0, 0, 53]]
